# hbfactor

#### 项目介绍
hb因子测试项目

#### 安装教程

```python
pip install hbfactor
```

#### 使用说明

```python
from hbfactor import FactorBase, get_factor
from sangreal_calendar import step_trade_dt


class DeltaRoa(FactorBase):
    # 继承FactorBase类 并覆写get_factor_value方法
    # 保证该方法返回的为因子截面数据，DataFrame结构，并存在sid及value 两列
    def get_factor_value(self, trade_dt):

        prev_dt = step_trade_dt(trade_dt, -252)
        # 如果使用get_factor方法，则默认从uqer拉取因子数据
        # 也可从wind底层库拉取数据合成因子
        roe0 = get_factor(factor_list=['ROA'], trade_dt=trade_dt)
        roe1 = get_factor(factor_list=['ROA'], trade_dt=prev_dt)
        delta_roe = (roe0 - roe1).reindex(roe0.index)
        delta_roe.reset_index(inplace=True)
        delta_roe.rename(columns={'roa': "value"}, inplace=True)
        return delta_roe


if __name__ == '__main__':
    # name 参数即为因子名，默认使用类名
    # path 参数即为因子存放位置，默认使用脚本相对路径
    delta = DeltaRoa(name=None, path=None)
    delta.update()
```

编写以上脚本 deltaroa.py，并执行

```python
python deltaroa.py
```

即可生成相应pandas的pk本地因子，用以下方式打开

```python
import pandas as pd
factor = pd.read_msgpack('delatroa.pk')
print(factor.head())

             sid  deltaroa  trade_dt
0      000001.SZ    0.0039  20080131
1      000002.SZ   -0.0219  20080131
2      000004.SZ    0.0942  20080131
3      000005.SZ    0.1158  20080131
4      000006.SZ   -0.0507  20080131
```

