import os
import pandas as pd
import datetime as dt

from abc import abstractmethod, ABCMeta
from sangreal_calendar import Monthly


class FactorBase(metaclass=ABCMeta):
    # 月末obj
    month = Monthly(-1)

    def __init__(self, name=None, path=None):
        """[目前只用来计算月末因子值]
        
        Keyword Arguments:
            metaclass {[type]} -- [description] (default: {ABCMeta):def__init__(self})
            name {[str]} -- [因子名称] (default: {None})
            path {[str]} -- [存放因子的msgpack文件] (default: {None})
        """

        if name is None:
            name = type(self).__name__.lower()
        self.name = name
        if path is None:
            path = f"{self.name}.pk"
        self.path = path
        self.date_back_to = self.month.prev('20030101')

    @staticmethod
    def uqer2wind(sid):
        return sid.replace('XSHE', 'SZ').replace('XSHG', 'SH')

    @abstractmethod
    def get_factor_value(self, trade_dt):
        """[计算trade_dt截面上的因子值，计算区间可由date_back_to决定]
        返回pd.DataFrame 三列 sid(个股id) value（因子值）
        Arguments:
            trade_dt {[str]} -- [section of factors]
        """

        pass

    def update(self):
        factor_pk = pd.DataFrame()
        while self.date_back_to < dt.date.today().strftime('%Y%m%d'):
            if os.path.exists(self.path):
                factor_pk = pd.read_msgpack(self.path)
                self.date_back_to = factor_pk.trade_dt.max()
            trade_dt = self.month.next(self.date_back_to)
            factor_pk_append = self.get_factor_value(trade_dt=trade_dt)

            factor_pk_append.dropna(inplace=True)
            if factor_pk_append.empty:
                self.date_back_to = trade_dt
                continue
            assert {'sid', 'value'}.issubset(
                factor_pk_append.columns), "Please check get_factor_value"
            factor_pk_append['trade_dt'] = trade_dt

            # 如果是uqer id 则进行转换
            check_id = factor_pk_append.sid.iloc[0]
            if 'XS' in check_id:
                factor_pk_append.sid = factor_pk_append.sid.map(self.uqer2wind)

            factor_pk_append.rename(columns={'value': self.name}, inplace=True)
            update_row = len(factor_pk_append)

            factor_pk = pd.concat([factor_pk, factor_pk_append],
                                  axis=0,
                                  ignore_index=True,
                                  sort=False)
            factor_pk.to_msgpack(self.path)
            print(f"{self.name}已更新{update_row}")
